#include <sigc++/bind.h>
#include <msp/time/utils.h>
#include "xineengine.h"
#include "xinema.h"
#include "xinestream.h"

using namespace std;
using namespace Msp;

Xinema::Xinema(int, char **):
	window(display, 1920, 1080),
	network(*this),
	engine(0),
	stream(0)
{
	window.signal_close.connect(sigc::bind(sigc::mem_fun(this, &Xinema::exit), 0));
}

int Xinema::main()
{
	window.show();
	display.tick();

	engine = new XineEngine(window, &display_mutex);

	Application::main();

	if(stream)
		delete stream;
	delete engine;

	return exit_code;
}

void Xinema::tick()
{
	{
		MutexLock lock(command_mutex);
		if(!pending_mrl.empty())
		{
			delete stream;
			stream = new XineStream(*engine, pending_mrl);
			stream->play();
			pending_mrl.clear();
		}
	}

	{
		MutexLock lock(display_mutex);
		display.tick();
	}

	engine->tick();
	if(stream)
		stream->tick();

	Time::sleep(10*Time::msec);
}

void Xinema::play_file(const FS::Path &fn)
{
	MutexLock lock(command_mutex);
	pending_mrl = "file://"+fn.str();
}
